function [cache count_score_cache] = build_score_cache(data_samples, parameters, count_score_cache)
% cache = build_score_cache(data_samples, parameters)
%
% build a cache for scoring DAGs.
% the cache contains the log-likelihood of each node for each parent set
% given data samples.
% The data_samples is an n x r matrix, in which n is the number of samples
% and r is the number of variables.
%
% CPDs integrated analytically, assuming a beta-bonomial (so variables assumed to be all binary).
%
% returns a cache structure:
%   cache.parent_vects = matrix of all possible parent vectors (in the
%                          order used in cache).
%   cache.vars{p} = list of variable indexes for parent set p.
%   cache.scores{p} = matrix containing score for each variable in
%                     cache.vars{p} on each prefix of the data indicated by
%                     parameters.score_data_at (1:parameters.score_data_at(i) for each i).
%                     has size: num_samples X num_variables.


hyper_p = parameters.hyperparameter;
num_vars = size(data_samples, 2);
num_samples = size(data_samples, 1);

num_prefixes = length(parameters.score_data_at);

%parent_vects contains all possible parent set vectors.
%assume that we go between index intengers and index vectors with bin_ind2vec
%and bin_vec2ind (which are inverses)...
parent_vects=bin_ind2vec( (1:(2^num_vars))',  num_vars );

cache.parent_vects=parent_vects;

%%switched to sparse count_score_cache, so don't need to extend size of
%%matrix.....
% if size(count_score_cache,1)<num_samples+1  %max counts are num_samples...
%     tmp=ones(num_samples+1,num_samples+1); 
%     %tmp=spalloc(num_samples+1,num_samples+1, nnz(count_score_cache)); %initialize to zeros...
%     tmp(1:size(count_score_cache,1), 1:size(count_score_cache,2)) = count_score_cache;
%     count_score_cache = tmp;
% end

normalizer=betaln(hyper_p(1), hyper_p(2));

for p=1:length(parent_vects)
    
    parent_vect=logical(parent_vects(p,:));
    
    %the variables which could have this parent set are all variables not
    %in the parent set:
    variables= ~parent_vect;
    cache.vars{p}=find(variables);
    
    scores=zeros(num_prefixes, sum(variables));
    
    % extract the information about the parents from the samples matrix
    parents_values = data_samples(:,parent_vect);
    parents_values_inds = bin_vec2ind(parents_values);
    
    %TODO: vectorize this loop.
    for prefix=1:num_prefixes
        
        % extract the information about the variable from the samples matrix
        variable_values = data_samples(1:parameters.score_data_at(prefix),variables);
        
        parents_values_ind = parents_values_inds(1:parameters.score_data_at(prefix));
        
        % since we don't know how many configurations there are to begin
        % with, we take the first one, extract it and repeat the process
        % for a shrunken matrix without those configurations.
        while ~isempty(parents_values_ind)
            
%             %clone the first row in the parent_values
%             clone = repmat(parents_values(1,:),size(parents_values,1),1);
%             
%             % find the location of configurations that match this one
%             % by comparing the clone to the original matrix
%             config_location = all(parents_values==clone,2);
            
            config_location = (parents_values_ind == parents_values_ind(1));
            
            scoring_variable_values=variable_values(config_location,:);
            
            %assume variables are binary, so just need to count the pos
            %(==1) and neg (==0 ==size - pos).
            pos = sum(scoring_variable_values,1);
            neg = size(scoring_variable_values,1)-pos;
            
            cache_positions = 1 + pos + neg*size(count_score_cache,1); %bare metal sub2ind...
            sc=count_score_cache(cache_positions);
            for v=1:length(pos)
                if (sc(v)==0)%(sc(v)==1)
                    sc(v) = betaln(hyper_p(1) + pos(v), hyper_p(2) + neg(v)) - normalizer;
                    count_score_cache(cache_positions(v))=sc(v);
                end
            end
            scores(prefix,:) = scores(prefix,:) + sc;
            
            %throw out the parent configurations we just went over
            %from the parent value matrix and the variable value matrix
%             parents_values = parents_values(~config_location,:);
            parents_values_ind = parents_values_ind(~config_location,:);
            variable_values = variable_values(~config_location,:);
            
        end
        
    end
    %now store these scores in the cache:
    cache.scores{p}=scores;
end
